<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              https://supporthost.com/
 * @since             1.0.0
 * @package           supporthost-star-rating
 *
 * @wordpress-plugin
 * Plugin Name:       Supporthost Star Rating
 * Plugin URI:        https://supporthost.com
 * Description:       This light plugin adds the possibility to give rating while commenting. Optimal for review websites.
 * Version:           1.0.0
 * Author:            SupportHost OU
 * Author URI:        https://supporthost.com/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       supporthost-star-rating
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

// load text domain
add_action( 'init', 'supporthost_star_rating_load_textdomain' );
function supporthost_star_rating_load_textdomain() {
    load_plugin_textdomain( 'supporthost-star-rating', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' ); 
}

function supporthost_star_rating_categories() {
    $options = get_option( 'supporthost_star_rating_plugin_options' );
    return $options['terms'];
}

function supporthost_star_rating_attributes() {
    $options = get_option( 'supporthost_star_rating_plugin_options' );
    $attributes =  explode(PHP_EOL, $options['attributes']);
    foreach ( $attributes as $attribute ) {
        $res[strtolower(trim($attribute))] = ucfirst(strtolower(trim($attribute))); 
    }
    return $res;
}

// Shows the stars with the comment form
add_action( 'comment_form_logged_in_after', 'supporthost_star_rating_rating_field' );
add_action( 'comment_form_after_fields', 'supporthost_star_rating_rating_field' );
function supporthost_star_rating_rating_field () {
	if ( in_category(supporthost_star_rating_categories()) ) {
		?>
		<label for="rating"><?php echo __('Rating', 'supporthost-star-rating') ?><span class="required">*</span></label>
		<fieldset class="supporthost-comments-rating">

				<?php
				$attributes = supporthost_star_rating_attributes();
				foreach ( $attributes as $key => $value) :
				?>
	<span class="supporthost-rating-container">
					<?php for ( $i = 5; $i >= 1; $i-- ) : ?>
						<input type="radio" id="<?php echo esc_attr($key); ?>-<?php echo esc_attr( $i ); ?>" name="<?php echo esc_attr($key); ?>" value="<?php echo esc_attr( $i ); ?>" /><label for="<?php echo esc_attr($key); ?>-<?php echo esc_attr( $i ); ?>"><?php echo esc_attr( $i ); ?></label>
					<?php endfor; ?>
					<input type="radio" id="<?php echo esc_attr($key); ?>-0" class="star-cb-clear" name="<?php echo esc_attr($key); ?>" value="0" /><label for="<?php echo esc_attr($key); ?>-0">0</label>
		<div class="rating-label"><?php echo esc_attr($value); ?></div>
	</span>
				<?php endforeach; ?>

		</fieldset>
		<?php
	}
}

//Save the rating submitted by the user.
add_action( 'comment_post', 'supporthost_star_rating_save_comment_rating' );
function supporthost_star_rating_save_comment_rating( $comment_id ) {
	$attributes = supporthost_star_rating_attributes();
	foreach ( $attributes as $key => $value) {
		if ( ( isset( $_POST[$key] ) ) && ( '' !== $_POST[$key] ) )
		$rating = intval( $_POST[$key] );
		add_comment_meta( $comment_id, $key, $rating );
	}
}

//Make the rating required.
add_filter( 'preprocess_comment', 'supporthost_star_rating_require_rating' );
function supporthost_star_rating_require_rating( $commentdata ) {

    
    if ( in_category(supporthost_star_rating_categories(), $_POST['comment_post_ID']) ) {
        $attributes = supporthost_star_rating_attributes();
		foreach ( $attributes as $key => $value) {
            if ( ! is_admin() && ( ! isset( $_POST[$key] ) || 0 === intval( $_POST[$key] ) ) ) {
				wp_die( __( 'Error: Please go back and select the number of stars for each section.' ) );
			}
		}
	}

	return $commentdata;
}

//Display the rating on a submitted comment.
add_filter( 'comment_text', 'supporthost_star_rating_display_rating', 10, 2);
function supporthost_star_rating_display_rating( $comment_text, $comment ){

    if ( $comment->comment_approved != 1 ) return $comment_text;

    $attributes = supporthost_star_rating_attributes();
	foreach ( $attributes as $key => $value) {
            $rating = get_comment_meta( $comment->comment_ID, $key, true );
            if ( !$rating ) continue;
			$stars = '<div class="stars"><span class="show-comment-label">'.$value.'</span><span class="show-stars">';
			for ( $i = 1; $i <= $rating; $i++ ) {
				$stars .= '<span class="dashicons dashicons-star-filled"></span>';
			}
			$stars .= '</span></div>';
			$comment_stars .= $stars;
		
	}
	return $comment_text . '<div class="supporthost-stars-container">' . $comment_stars . '</div>';
}

//Get the average rating of a post.
function supporthost_star_rating_get_average_ratings( $id, $key ) {
	$comments = get_approved_comments( $id );

	if ( $comments ) {
		$i = 0;
		$total = 0;
		foreach( $comments as $comment ){
			$rate = get_comment_meta( $comment->comment_ID, $key, true );
			if( isset( $rate ) && '' !== $rate ) {
				$i++;
				$total += $rate;
			}
		}

		if ( 0 === $i ) {
			return false;
		} else {
			return round( $total / $i, 1 );
		}
	} else {
		return false;
	}
}

//Display the average rating above the content.
add_filter( 'the_content', 'supporthost_star_rating_display_average_rating' );
function supporthost_star_rating_display_average_rating( $content ) {

    $options = get_option( 'supporthost_star_rating_plugin_options' );
    if ( isset($options['show_on_top']) && $options['show_on_top'] == 'no' ) {
        return $content;
    }
    
	$custom_content .= supporthost_star_rating_shortcode() . $content;

	if ( is_single() ) {
		return $custom_content;
	} else {
		return $content;
	}
}

//shortcode [supporthost-average-rating]
add_shortcode('supporthost-average-rating', 'supporthost_star_rating_shortcode');
function supporthost_star_rating_shortcode(){

    global $post;
	
	$attributes = supporthost_star_rating_attributes();
	foreach ( $attributes as $key => $value) {
		$stars[$key]   = '';
		$average[$key] = supporthost_star_rating_get_average_ratings( $post->ID, $key );

		for ( $i = 1; $i <= $average[$key] + 1; $i++ ) {

			$width[$key] = intval( $i - $average[$key] > 0 ? 20 - ( ( $i - $average[$key] ) * 20 ) : 20 );

			if ( 0 === $width[$key] ) {
				continue;
			}

			$stars[$key] .= '<span style="overflow:hidden; width:' . $width[$key] . 'px" class="dashicons dashicons-star-filled"></span>';

			if ( $i - $average[$key] > 0 ) {
				$stars[$key] .= '<span style="overflow:hidden; position:relative; left:-' . $width[$key] .'px;" class="dashicons dashicons-star-half"></span>';
			}
		}
	}
	$attributes = supporthost_star_rating_attributes();
	$custom_content  = '<div class="supporthost-average-rating-box">';
	foreach ( $attributes as $key => $value) {
        if ( !$average[$key] ) {
            $custom_content  .= '<div class="average-rating-single"><span class="average-rating-label">' . $value . '</span><span class="average-rating-value">' . __('No rating yet, be the first to rate this!', 'supporthost-star-rating') .'</span></div>';
        } else {
            $custom_content  .= '<div class="average-rating-single"><span class="average-rating-label">' . $value . '</span><span class="average-rating-value">' . $average[$key] .'</span> <span class="average-rating-stars">' . $stars[$key] .'</span></div>';
        }
	}
	$custom_content .= '</div>';

    return $custom_content;
}

// enqueue scripts/css
function supporthost_star_rating_scripts() {
    wp_enqueue_style( 'ssr', plugin_dir_url( __FILE__ ) . 'css/supporthost-star-rating.css', array(), '1.0.0', 'all' );
    wp_enqueue_style('dashicons');
}
add_action( 'wp_enqueue_scripts', 'supporthost_star_rating_scripts' );




// Plugin settings
function supporthost_star_rating_add_settings_page() {
    add_options_page( 'SupportHost star rating plugin', 'SupportHost star rating', 'manage_options', 'supporthost-star-rating-plugin', 'supporthost_star_rating_render_plugin_settings_page' );
}
add_action( 'admin_menu', 'supporthost_star_rating_add_settings_page' );


function supporthost_star_rating_render_plugin_settings_page() {
    ?>
    <!-- <div style="
    background-color: #fff;
    border-color: #d0d7de;
    border-radius: 10px;
    border-style: solid;
    border-width: 1px;
    margin-top: 25px;
    max-width: 90%;
    "> -->
        <h1 style="
    background-color: #f6f8fa;
    border-color: #d0d7de;
    border-style: solid;
    border-top-left-radius: 10px;
    border-top-right-radius: 10px;
    border-width: 1px;
    padding: 25px;
    margin-top: 25px;
    max-width: 90%;
    margin-bottom: 0;
        "><?php _e('Sponsor', 'supporthost-star-rating') ?></h1>
        <p style="
        font-size: 20px;
        line-height: 1.5;
        margin-bottom:50px;
        background-color: #fff;
        border-color: #d0d7de;
        border-style: solid;
        border-bottom-left-radius: 10px;
        border-bottom-right-radius: 10px;
        margin-top: -1px;
        padding: 25px;
        max-width: 90%;


        
        "><?php _e('Not happy with your web hosting? Get a professional service with <a href="https://supporthost.com/?promocode=star-rating">SupportHost at 30% discount</a> for the first year. 30 days guarantee and <a href="https://supporthost.com/free-hosting/">14 days free trial!</a>', 'supporthost-star-rating') ?></p>
    <!-- </div> -->
    <form action="options.php" method="post">
            <?php 
        settings_fields( 'supporthost_star_rating_plugin_options' );
        do_settings_sections( 'supporthost_star_rating_plugin' ); ?>
        <input name="submit" class="button button-primary" type="submit" value="<?php esc_attr_e( 'Save' ); ?>" />
    </form>
    <?php
}


function supporthost_star_rating_register_settings() {
    register_setting( 'supporthost_star_rating_plugin_options', 'supporthost_star_rating_plugin_options', 'supporthost_star_rating_plugin_options_validate' );
    add_settings_section( 'supporthost_star_rating_settings', __('SupportHost star rating plugin Settings', 'supporthost-star-rating'), 'supporthost_star_rating_section_text', 'supporthost_star_rating_plugin' );

    add_settings_field( 'supporthost_star_rating_setting_show_on_top', __('Show rating on top', 'supporthost-star-rating'), 'supporthost_star_rating_setting_show_on_top', 'supporthost_star_rating_plugin', 'supporthost_star_rating_settings' );
    add_settings_field( 'supporthost_star_rating_setting_terms', __('Select terms', 'supporthost-star-rating'), 'supporthost_star_rating_setting_terms', 'supporthost_star_rating_plugin', 'supporthost_star_rating_settings' );
    add_settings_field( 'supporthost_star_rating_setting_attributes', __('Attributes', 'supporthost-star-rating'), 'supporthost_star_rating_setting_attributes', 'supporthost_star_rating_plugin', 'supporthost_star_rating_settings' );
}
add_action( 'admin_init', 'supporthost_star_rating_register_settings' );


function supporthost_star_rating_plugin_options_validate( $input ) {

    // checks if selected terms are real terms in the database
    $taxonomies = supporthost_star_rating_availlabe_taxonomies();
    $terms = [];
    foreach ( get_terms($taxonomies) as $term ) {
        $terms[] = $term->slug;
    }
    $newinput['terms'] = array_intersect($terms, $input['terms']);

    // trim attributes, sanitizes and remove new lines
    $newinput['attributes'] = trim( sanitize_textarea_field( $input['attributes'] ) );
    $newinput['attributes'] = preg_replace('/^\h*\v+/m', '', $newinput['attributes']);

    // sanitizes the checkbox
    $newinput['show_on_top'] = ( $input['show_on_top'] == 'on') ? 'yes' : 'no';

    return $newinput;
}

function supporthost_star_rating_section_text() {
    echo '<p>' . __('Here you can set all the options for the star rating plugin', 'supporthost-star-rating') . '</p>';
    $options = get_option( 'supporthost_star_rating_plugin_options' );
}

function supporthost_star_rating_setting_terms() {
    $taxonomies = supporthost_star_rating_availlabe_taxonomies();

    $options = get_option( 'supporthost_star_rating_plugin_options' );

    echo '<select id="supporthost_star_rating_setting_terms" name="supporthost_star_rating_plugin_options[terms][]" multiple>';

    foreach ( get_terms($taxonomies) as $term ) {

        $terms = ( is_array($options['terms']) ) ? $options['terms'] : [];

        echo '<option value="' . esc_attr($term->slug) . '"';
        // check if selected
        if ( in_array($term->slug, $terms) ) {
            echo ' selected';
        }
        echo '>' . esc_attr($term->slug) . ' (' . esc_attr($term->taxonomy) . ')</option>';

    }

    echo '</select>';
    echo '<p>' . __('Select the terms for which you want to show the star rating', 'supporthost-star-rating') . '</p>';
}

function supporthost_star_rating_setting_show_on_top() {
    $options = get_option( 'supporthost_star_rating_plugin_options' );
    echo '<input type="checkbox" id="show_on_top" name="supporthost_star_rating_plugin_options[show_on_top]"';
    if ( isset($options['show_on_top']) && $options['show_on_top'] == 'yes' ) {
        echo ' checked';
    }
    echo'><label for="show_on_top"> ' . __('Select this to show an aggregate rating on top of every content that can be rated', 'supporthost-star-rating') . '</label><br>';
}

function supporthost_star_rating_setting_attributes() {
    $options = get_option( 'supporthost_star_rating_plugin_options' );
    echo '<textarea id="supporthost_star_rating_setting_attributes" name="supporthost_star_rating_plugin_options[attributes]" rows="4" cols="50">';
    echo esc_attr($options['attributes']);
    echo '</textarea>';
    echo '<p>' . __('Add a list of attributes you want to ask a rating for, one per line', 'supporthost-star-rating') . '</p>';
}

function supporthost_star_rating_availlabe_taxonomies() {
    return array_diff(get_taxonomies(), array('wp_template_part_area', 'wp_theme', 'post_format', 'link_category', 'nav_menu') );
}
// End Plugin settings